@@ -47,6 +47,7 @@ struct MappingOptions {
     bool decode_blessed;
     bool fail_ref_coercion;
     bool numeric_bool;
+    bool no_redefine_perl_names;
     AccessorStyle accessor_style;
     ClientService client_services;
 
@@ -72,6 +73,7 @@ public:
     void map_message_prefix(pTHX_ const std::string &message, const std::string &perl_package_prefix, const MappingOptions &options);
     void map_enum(pTHX_ const std::string &enum_name, const std::string &perl_package, const MappingOptions &options);
     void map_service(pTHX_ const std::string &service_name, const std::string &perl_package, const MappingOptions &options);
+    void map_wkts(pTHX_ const MappingOptions &options);
     void resolve_references();
 
     const Mapper *find_mapper(const upb::MessageDef *message_def) const;
@@ -85,10 +87,12 @@ public:
     }
 
 private:
+    void add_file_recursively(pTHX_ const google::protobuf::FileDescriptor *file);
     void map_package_or_prefix(pTHX_ const std::string &pb_package, bool is_prefix, const std::string &perl_package_prefix, const MappingOptions &options);
     void map_message_recursive(pTHX_ const google::protobuf::Descriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
     void map_message_prefix_recursive(pTHX_ const google::protobuf::Descriptor *descriptor, const std::string &perl_package_prefix, const MappingOptions &options, STD_TR1::unordered_set<std::string> &recursed_names);
     void map_message(pTHX_ const google::protobuf::Descriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
+    void bind_message(pTHX_ const std::string &perl_package, Mapper *mapper, HV *stash, const MappingOptions &options);
     void map_enum(pTHX_ const google::protobuf::EnumDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
     void map_service(pTHX_ const google::protobuf::ServiceDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options);
     void map_service_noop(pTHX_ const google::protobuf::ServiceDescriptor *descriptor, const std::string &perl_package, const MappingOptions &options, ServiceDef *service_def);
